﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;
using MVR.FileManagementSecure;
using UnityEngine.UI;

namespace JayJayWon
{
    public class Settings
    {

        public static JSONStorableFloat maxButtonsPerPageJSF = new JSONStorableFloat("Max Resources Per Page",200f,MaxButtonsPerPageCallback, 20f,2000f);
        public static JSONStorableFloat uiScaleJSF = new JSONStorableFloat("Desktop UI Scale", 1f, UIScaleCallback, 0.5f, 2f);
        public static JSONStorableEnumStringChooser resourceNameSearchModeJSESC = new JSONStorableEnumStringChooser("resourceNameSearchMode", ResourceNameSearchMode.enumManifestName,ResourceNameSearchMode.ContainsFullPath,"Resource Name Filter Mode", ResourceNameSearchModeCallback);
        public static JSONStorableEnumStringChooser launchButtonPlacementJSESC = new JSONStorableEnumStringChooser("launchButtonPlacement", VAMUILaunchButtonPosition.enumManifestName, VAMUILaunchButtonPosition.toolBarGap, "Launch Button Position", LaunchButtonPlacementCallback);

        public static Dictionary<int, ResourceSettings> resourceSettings = new Dictionary<int, ResourceSettings>();

        private static bool restoreInProgress = false;

        public Settings()
        {
            
        }

        private static void LaunchButtonPlacementCallback(string val)
        {
            SaveSettings();
            if (!restoreInProgress) Globals.mvrScript.RePositionLaunchButton();
        }
        private static void MaxButtonsPerPageCallback(float val)
        {
            maxButtonsPerPageJSF.valNoCallback = Mathf.Round(val / 10f) *10f;
            if (BrowserAssist.browserUI!=null && BrowserAssist.browserUI.resourceBrowserUI != null && BrowserAssist.browserUI.isUIActive) BrowserAssist.browserUI.RefreshButtonLayout();
            SaveSettings();
        }

        private static void ResourceNameSearchModeCallback(string val)
        {
            if (BrowserAssist.browserUI != null) BrowserAssist.browserUI.browserFiltersUI.SetResourceNameFilterHelpText();
            SaveSettings();
        }
        private static void UIScaleCallback(float val)
        {
            if (BrowserAssist.browserUI != null) BrowserAssist.browserUI.RefreshScale();
            SaveSettings();
        }
        private static void SettingsBoolChange(bool val)
        {
            SaveSettings();
        }

        private static void SettingsFloatChange(float val)
        {
            SaveSettings();
        }

        private static void SettingsStringChange(string val)
        {
            SaveSettings();
        }

        public static void SaveSettings()
        {
            if (!restoreInProgress)
            {
                JSONClass settingsJC = new JSONClass();

                maxButtonsPerPageJSF.StoreJSON(settingsJC["BrowserSettings"].AsObject);
                uiScaleJSF.StoreJSON(settingsJC["DesktopSettings"].AsObject);
                resourceNameSearchModeJSESC.StoreJSON(settingsJC["FilterSettings"].AsObject);
                launchButtonPlacementJSESC.StoreJSON(settingsJC["GeneralSettings"].AsObject);

                foreach (int resourceType in EnumManifest.GetEnumsList(ResourceType.enumManifestName))
                {
                    string resourceTypeStoreLabel = EnumManifest.GetStoreVal(ResourceType.enumManifestName, resourceType);

                    settingsJC["ResourceSettings"][resourceTypeStoreLabel] = resourceSettings[resourceType].GetJSON();
                }

                SuperController.singleton.SaveJSON(settingsJC, Const.BAScriptDataFolder + "\\" + "BASettings.cfg");
            }

        }

        public static void LoadSettings()
        {
            if (FileManagerSecure.FileExists(Const.BAScriptDataFolder + "\\" + "BASettings.cfg"))
            {
                
                JSONClass settingsJC;
                try
                {
                    settingsJC = SuperController.singleton.LoadJSON(Const.BAScriptDataFolder + "\\" + "BASettings.cfg").AsObject;
                }
                catch
                {
                    Utils.LogDebug("Settings.LoadSettings: Error during load of settings config file. Possible malformed JSON");
                    return;
                }

                restoreInProgress = true;

                foreach (int resourceType in EnumManifest.GetEnumsList(ResourceType.enumManifestName))
                {
                    ResourceSettings newResourceSettings = new ResourceSettings();
                    
                    string resourceTypeStoreLabel = EnumManifest.GetStoreVal(ResourceType.enumManifestName, resourceType);

                    newResourceSettings.RestoreJSON(settingsJC["ResourceSettings"][resourceTypeStoreLabel].AsObject);

                    resourceSettings.Add(resourceType, newResourceSettings);
                }

                uiScaleJSF.RestoreFromJSON(settingsJC["DesktopSettings"].AsObject);
                maxButtonsPerPageJSF.RestoreFromJSON(settingsJC["BrowserSettings"].AsObject);
                resourceNameSearchModeJSESC.RestoreFromJSON(settingsJC["FilterSettings"].AsObject);
                launchButtonPlacementJSESC.RestoreFromJSON(settingsJC["GeneralSettings"].AsObject);

                restoreInProgress = false;
            }
            else
            {
                foreach (int resourceType in EnumManifest.GetEnumsList(ResourceType.enumManifestName))
                {
                    ResourceSettings newResourceSettings = new ResourceSettings();
                    resourceSettings.Add(resourceType, newResourceSettings);
                }
                SaveSettings();
            }

        }
    }

    public class ResourceSettings
    {
        public JSONStorableBool closeBrowserUIonResourceLoadJSB = new JSONStorableBool("closeBrowserUIonResourceLoad", false, SettingsBoolChange);
        public ResourceSettings()
        {

        }

        public virtual JSONClass GetJSON()
        {
            JSONClass resourceSettingsJC = new JSONClass();
            closeBrowserUIonResourceLoadJSB.StoreJSON(resourceSettingsJC);

            return resourceSettingsJC;
        }

        public virtual void RestoreJSON(JSONClass resourceSettingsJC)
        {
            closeBrowserUIonResourceLoadJSB.RestoreFromJSON(resourceSettingsJC);
        }

        private static void SettingsBoolChange(bool val)
        {
            Settings.SaveSettings();
        }
    }
}
